@@ -378,6 +378,8 @@ on_monitors_changed (GdkScreen *gdk_screen, gpointer user_data)
 
     screen = CS_SCREEN (user_data);
 
+    gdk_flush ();
+
     old_monitor_infos = screen->monitor_infos;
 
     reload_monitor_infos (screen);
@@ -394,6 +396,8 @@ on_screen_changed (GdkScreen *gdk_screen, gpointer user_data)
 
     screen = CS_SCREEN (user_data);
 
+    gdk_flush ();
+
     reload_screen_info (screen);
 
     g_signal_emit (screen, signals[SCREEN_CHANGED], 0);
@@ -627,6 +631,20 @@ cs_screen_get_mouse_monitor (CsScreen *screen)
     return ret;
 }
 
+/**
+ * cs_screen_reset_screensaver:
+ *
+ * Resets the screensaver idle timer. If called when the screensaver is active
+ * it will stop it.
+ *
+ */
+
+void
+cs_screen_reset_screensaver (void)
+{
+    XResetScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
+}
+
 void
 cs_screen_nuke_focus (void)
 {
