@@ -49,13 +49,13 @@ class Stage(Gtk.Window):
         self.get_style_context().add_class("csstage")
 
         trackers.con_tracker_get().connect(singletons.Backgrounds,
-                                           "changed", 
+                                           "changed",
                                            self.on_bg_changed)
 
         self.destroying = False
 
         self.manager = manager
-        self.screen = CScreensaver.Screen.new(status.Debug)
+        status.screen = CScreensaver.Screen.new(status.Debug)
         self.away_message = away_message
 
         self.monitors = []
@@ -118,9 +118,13 @@ class Stage(Gtk.Window):
         # For instance: Chrome and Firefox native notifications.
         self.gdk_filter = CScreensaver.GdkEventFilter()
 
-        trackers.con_tracker_get().connect(self.screen,
-                                           "changed",
-                                           self.on_screen_changed)
+        trackers.con_tracker_get().connect(status.screen,
+                                           "size-changed",
+                                           self.on_screen_size_changed)
+
+        trackers.con_tracker_get().connect(status.screen,
+                                           "monitors-changed",
+                                           self.on_monitors_changed)
 
         trackers.con_tracker_get().connect(self,
                                            "grab-broken-event",
@@ -129,13 +133,34 @@ class Stage(Gtk.Window):
         if status.InteractiveDebug:
             self.set_interactive_debugging(True)
 
-    def on_screen_changed(self, screen, data=None):
+    def update_monitors(self):
+        self.destroy_monitor_views()
+
+        try:
+            self.setup_monitors()
+            for monitor in self.monitors:
+                self.sink_child_widget(monitor)
+        except Exception as e:
+            print("Problem updating monitor views views: %s" % str(e))
+
+    def on_screen_size_changed(self, screen, data=None):
+        if status.Debug:
+            print("Stage: Received screen changed signal, updating backdrop")
+
+        Gdk.flush()
+
         self.update_geometry()
         self.move_onscreen()
 
-        for monitor in self.monitors:
-            monitor.update_geometry()
+        self.overlay.queue_resize()
 
+    def on_monitors_changed(self, screen, data=None):
+        if status.Debug:
+            print("Stage: Received screen monitors-changed signal, updating monitor views")
+
+        Gdk.flush()
+
+        self.update_monitors()
         self.overlay.queue_resize()
 
     def on_grab_broken_event(self, widget, event, data=None):
@@ -143,10 +168,22 @@ class Stage(Gtk.Window):
 
         return False
 
+    def refresh(self):
+        Gdk.flush()
+
+        self.update_geometry()
+        self.move_onscreen()
+        self.update_monitors()
+        self.overlay.queue_resize()
+
     def transition_in(self, effect_time, callback):
         """
         This is the primary way of making the Stage visible.
         """
+
+        # Cancel any existing transition
+        self.fader.cancel()
+
         if effect_time == 0:
             self.set_opacity(1.0)
             self.move_onscreen()
@@ -264,12 +301,14 @@ class Stage(Gtk.Window):
 
         try:
             if self.clock_widget != None:
+                self.clock_widget.stop_positioning()
                 self.clock_widget.destroy()
         except Exception as e:
             print(e)
 
         try:
             if self.albumart_widget != None:
+                self.albumart_widget.stop_positioning()
                 self.albumart_widget.destroy()
         except Exception as e:
             print(e)
@@ -322,11 +361,20 @@ class Stage(Gtk.Window):
         self.gdk_filter.stop()
         self.gdk_filter = None
 
-        trackers.con_tracker_get().disconnect(self.screen,
-                                              "changed",
-                                              self.on_screen_changed)
+        trackers.con_tracker_get().disconnect(status.screen,
+                                              "size-changed",
+                                              self.on_screen_size_changed)
+
+        trackers.con_tracker_get().disconnect(status.screen,
+                                              "monitors-changed",
+                                              self.on_monitors_changed)
+
+        trackers.con_tracker_get().disconnect(self.overlay,
+                                              "get-child-position",
+                                              self.position_overlay_child)
 
         self.destroy()
+        status.screen = None
 
     def setup_monitors(self):
         """
@@ -337,15 +385,15 @@ class Stage(Gtk.Window):
         status.Spanned = settings.bg_settings.get_enum("picture-options") == CDesktopEnums.BackgroundStyle.SPANNED
 
         if status.InteractiveDebug or status.Spanned:
-            monitors = (self.screen.get_primary_monitor(),)
+            monitors = (status.screen.get_primary_monitor(),)
         else:
-            n = self.screen.get_n_monitors()
+            n = status.screen.get_n_monitors()
             monitors = ()
             for i in range(n):
                 monitors += (i,)
 
         for index in monitors:
-            monitor = MonitorView(self.screen, index)
+            monitor = MonitorView(index)
 
             image = Gtk.Image()
 
@@ -370,8 +418,8 @@ class Stage(Gtk.Window):
             image = Gtk.Image()
 
             singletons.Backgrounds.create_and_set_gtk_image (image,
-                                                  monitor.rect.width,
-                                                  monitor.rect.height)
+                                                             monitor.rect.width,
+                                                             monitor.rect.height)
 
             monitor.set_next_wallpaper_image(image)
 
@@ -388,6 +436,9 @@ class Stage(Gtk.Window):
                                            "current-view-change-complete",
                                            self.after_power_state_changed)
 
+        if status.Debug:
+            print("stage: Power state changed, updating monitor views")
+
         self.update_monitor_views()
 
     def after_power_state_changed(self, monitor):
@@ -409,12 +460,12 @@ class Stage(Gtk.Window):
         Initially invisible, regardless - its visibility is controlled via its
         own positioning timer.
         """
-        self.clock_widget = ClockWidget(self.screen, self.away_message, self.screen.get_mouse_monitor())
+        self.clock_widget = ClockWidget(self.away_message, status.screen.get_mouse_monitor())
         self.add_child_widget(self.clock_widget)
 
         self.floaters.append(self.clock_widget)
 
-        if (not settings.should_show_plugin()) or (settings.should_show_plugin() and not self.power_client.plugged_in):
+        if not status.shouldShowPlugin():
             if settings.get_show_clock():
                 self.clock_widget.start_positioning()
 
@@ -427,12 +478,12 @@ class Stage(Gtk.Window):
         Initially invisible, regardless - its visibility is controlled via its
         own positioning timer.
         """
-        self.albumart_widget = AlbumArt(self.screen, self.away_message, self.screen.get_mouse_monitor())
+        self.albumart_widget = AlbumArt(None, status.screen.get_mouse_monitor())
         self.add_child_widget(self.albumart_widget)
 
         self.floaters.append(self.clock_widget)
 
-        if (not settings.should_show_plugin()) or (settings.should_show_plugin() and not self.power_client.plugged_in):
+        if not status.shouldShowPlugin():
             if settings.get_show_albumart():
                 self.albumart_widget.start_positioning()
 
@@ -478,10 +529,10 @@ class Stage(Gtk.Window):
         """
         Constructs the AudioPanel and InfoPanel and adds them to the overlay.
         """
-        self.audio_panel = AudioPanel(self.screen)
+        self.audio_panel = AudioPanel()
         self.add_child_widget(self.audio_panel)
 
-        self.info_panel = InfoPanel(self.screen)
+        self.info_panel = InfoPanel()
         self.add_child_widget(self.info_panel)
 
     def queue_dialog_key_event(self, event):
@@ -613,7 +664,7 @@ class Stage(Gtk.Window):
         """
         Hide the unlock widget (and others) if the unlock has been canceled
 
-        This process is in three steps for aesthetic reasons - 
+        This process is in three steps for aesthetic reasons -
             a) Unreveal all widgets (begin fading them out)
             b) Switch over MonitorViews from wallpaper to plug-ins if needed.
             c) Re-reveal the InfoPanel if applicable
@@ -681,23 +732,22 @@ class Stage(Gtk.Window):
         Updates all of our MonitorViews based on the power
         or Awake states.
         """
-        low_power = not self.power_client.plugged_in
-
-        if (not settings.should_show_plugin()) or (settings.should_show_plugin() and low_power):
-            if self.clock_widget != None and settings.get_show_clock():
-                self.clock_widget.start_positioning()
-            if self.albumart_widget != None and settings.get_show_albumart():
-                self.albumart_widget.start_positioning()
-        else:
-            if self.clock_widget != None:
-                self.clock_widget.stop_positioning()
-                self.clock_widget.hide()
-            if self.albumart_widget != None:
-                self.albumart_widget.stop_positioning()
-                self.albumart_widget.hide()
+        if not status.Awake:
+            if (not status.shouldShowPlugin()):
+                if self.clock_widget != None and settings.get_show_clock():
+                    self.clock_widget.start_positioning()
+                if self.albumart_widget != None and settings.get_show_albumart():
+                    self.albumart_widget.start_positioning()
+            else:
+                if self.clock_widget != None:
+                    self.clock_widget.stop_positioning()
+                    self.clock_widget.hide()
+                if self.albumart_widget != None:
+                    self.albumart_widget.stop_positioning()
+                    self.albumart_widget.hide()
 
         for monitor in self.monitors:
-            monitor.update_view(status.Awake, low_power)
+            monitor.update_view()
 
             if not monitor.get_reveal_child():
                 monitor.reveal()
@@ -736,10 +786,13 @@ class Stage(Gtk.Window):
         """
 
         if status.InteractiveDebug:
-            monitor_n = self.screen.get_primary_monitor()
-            self.rect = self.screen.get_monitor_geometry(monitor_n)
+            monitor_n = status.screen.get_primary_monitor()
+            self.rect = status.screen.get_monitor_geometry(monitor_n)
         else:
-            self.rect = self.screen.get_screen_geometry()
+            self.rect = status.screen.get_screen_geometry()
+
+        if status.Debug:
+            print("Stage.update_geometry - new backdrop position: %d, %d  new size: %d x %d" % (self.rect.x, self.rect.y, self.rect.width, self.rect.height))
 
         hints = Gdk.Geometry()
         hints.min_width = self.rect.width
@@ -755,16 +808,16 @@ class Stage(Gtk.Window):
 
     def get_mouse_monitor(self):
         if status.InteractiveDebug:
-            return self.screen.get_primary_monitor()
+            return status.screen.get_primary_monitor()
         else:
-            return self.screen.get_mouse_monitor()
+            return status.screen.get_mouse_monitor()
 
     def maybe_update_layout(self):
         """
         Called on all user events, moves widgets to the currently
         focused monitor if it changes (whichever monitor the mouse is in)
         """
-        current_focus_monitor = self.screen.get_mouse_monitor()
+        current_focus_monitor = status.screen.get_mouse_monitor()
 
         if self.last_focus_monitor == -1:
             self.last_focus_monitor = current_focus_monitor
@@ -780,6 +833,12 @@ class Stage(Gtk.Window):
         """
         self.overlay.add_overlay(widget)
 
+    def sink_child_widget(self, widget):
+        """
+        Move a child to the bottom of the overlay
+        """
+        self.overlay.reorder_overlay(widget, 0)
+
     def position_overlay_child(self, overlay, child, allocation):
         """
         Callback for our GtkOverlay, think of this as a mini-
@@ -814,8 +873,8 @@ class Stage(Gtk.Window):
             UnlockDialog always shows on the currently focused monitor (the one the
             mouse is currently in), and is kept centered.
             """
-            monitor = self.screen.get_mouse_monitor()
-            monitor_rect = self.screen.get_monitor_geometry(monitor)
+            monitor = status.screen.get_mouse_monitor()
+            monitor_rect = status.screen.get_monitor_geometry(monitor)
 
             min_rect, nat_rect = child.get_preferred_size()
 
@@ -840,11 +899,11 @@ class Stage(Gtk.Window):
             min_rect, nat_rect = child.get_preferred_size()
 
             if status.Awake:
-                current_monitor = self.screen.get_mouse_monitor()
+                current_monitor = status.screen.get_mouse_monitor()
             else:
                 current_monitor = child.current_monitor
 
-            monitor_rect = self.screen.get_monitor_geometry(current_monitor)
+            monitor_rect = status.screen.get_monitor_geometry(current_monitor)
 
             region_w = monitor_rect.width / 3
             region_h = monitor_rect.height / 3
@@ -927,8 +986,8 @@ class Stage(Gtk.Window):
             min_rect, nat_rect = child.get_preferred_size()
 
             if status.Awake:
-                current_monitor = self.screen.get_mouse_monitor()
-                monitor_rect = self.screen.get_monitor_geometry(current_monitor)
+                current_monitor = status.screen.get_mouse_monitor()
+                monitor_rect = status.screen.get_monitor_geometry(current_monitor)
                 allocation.x = monitor_rect.x
                 allocation.y = monitor_rect.y
                 allocation.width = nat_rect.width
@@ -952,8 +1011,8 @@ class Stage(Gtk.Window):
             min_rect, nat_rect = child.get_preferred_size()
 
             if status.Awake:
-                current_monitor = self.screen.get_mouse_monitor()
-                monitor_rect = self.screen.get_monitor_geometry(current_monitor)
+                current_monitor = status.screen.get_mouse_monitor()
+                monitor_rect = status.screen.get_monitor_geometry(current_monitor)
                 allocation.x = monitor_rect.x + monitor_rect.width - nat_rect.width
                 allocation.y = monitor_rect.y
                 allocation.width = nat_rect.width
